﻿namespace VistaDBQuickAndDirtyBrowser
{
	using System;
	using System.Data;
	using System.Drawing;
	using System.Windows.Forms;

	public partial class DetailsForm :
		Form
	{
		private DataRow _row;
		private string _tableName;
		private MainForm _mainForm;
		private static int _defaultWindowWidth;
		private static int _defaultWindowHeight;
		private Action<int> _writeToDB;

		public DetailsForm()
		{
			InitializeComponent();
		}

		public void Initialize(DataRow row, string tableName, MainForm mainForm, Action<int> writeToDB )
		{
			_row = row;
			_tableName = tableName;
			_mainForm = mainForm;
			_writeToDB = writeToDB;
		}

		private void DetailsForm_Load(object sender, EventArgs e)
		{
			if (_defaultWindowWidth > 0) Width = _defaultWindowWidth;
			//if (_defaultWindowHeight > 0) Height = _defaultWindowHeight;

			buildControls();
			CenterToParent();

			fillObjectToControls();
		}

		private void DetailsForm_FormClosing(object sender, FormClosingEventArgs e)
		{
			_defaultWindowWidth = Width;
			_defaultWindowHeight = Height;
		}

		private void buttonOK_Click(object sender, EventArgs e)
		{
			try
			{
				fillControlsToObject();
				_writeToDB(0);
			}
			catch (Exception)
			{
				DialogResult = DialogResult.None;
				throw;
			}
		}

		private class FillStateInfo
		{
			public int CurrentY { get; set; }
			public int WidestLabelWidth { get; set; }
			public int LabelDistance { get { return 2; } }
			public int YDistance { get { return 5; } }

			public int TabIndex { get; set; }
		}

		private void buildControls()
		{
			var table = _row.Table;

			var fsi = new FillStateInfo { CurrentY = panel1.Padding.Top };

			// Measure labels.
			foreach (DataColumn column in table.Columns)
			{
				fsi.WidestLabelWidth = Math.Max(fsi.WidestLabelWidth, measureWidth(column.ColumnName + @":"));
			}

			foreach (DataColumn column in table.Columns)
			{
				doBuildControl(fsi, column);
			}

			// --

			var delta = fsi.CurrentY - buttonOK.Top + panel1.Padding.Bottom;
			Height += delta;

			MinimumSize = new Size(MinimumSize.Width, Height);
		}

		private int measureWidth(string s)
		{
			using (var g = CreateGraphics())
			{
				return (int)g.MeasureString(s, Font).Width;
			}
		}

		private void doBuildControl(FillStateInfo fsi, DataColumn column)
		{
			var x = panel1.Padding.Left;

			if (column.DataType != typeof(bool))
			{
				var label = new Label
								{
									Font = Font,
									Text = column.ColumnName + @":",
									Left = x,
									Top = fsi.CurrentY,
									Tag = column,
									AutoSize = true,
									TabIndex = fsi.TabIndex++
								};
				panel1.Controls.Add(label);
			}

			Control control;

			if (column.DataType == typeof(bool))
			{
				control = new CheckBox { Text = column.ColumnName, AutoSize = true };
			}
			else if (column.DataType == typeof(int) || column.DataType == typeof(short))
			{
				control = new TextBox
				{
					Width = 100,
					Anchor = AnchorStyles.Left | AnchorStyles.Top
				};
			}
			else if (column.DataType == typeof(string))
			{
				var tb = new TextBox
							{
								Width =
								panel1.Width - panel1.Padding.Horizontal - (x + fsi.WidestLabelWidth /*+ fsi.LabelDistance*/),
								Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right
							};

				if (isMultiLine(column))
				{
					tb.Multiline = true;
					tb.Height = 150;
					tb.ScrollBars = ScrollBars.Vertical;
					tb.AcceptsReturn = true;
				}

				control = tb;
			}
			else
			{
				control = new TextBox
							{
								Width =
								panel1.Width - panel1.Padding.Horizontal - (x + fsi.WidestLabelWidth /*+ fsi.LabelDistance*/),
								Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right
							};
			}

			control.Left = x + fsi.WidestLabelWidth + fsi.LabelDistance;
			control.Top = fsi.CurrentY;
			control.Tag = column;
			control.TabIndex = fsi.TabIndex++;
			panel1.Controls.Add(control);

			fsi.CurrentY += control.Height + fsi.YDistance;
		}

		private bool isMultiLine(DataColumn column)
		{
			if (column.DataType == typeof(string))
			{
				var cn = _mainForm.GetTableColumnType(_tableName, column.ColumnName);

				return
					//string.Compare(cn, @"nvarchar", StringComparison.InvariantCultureIgnoreCase) == 0 ||
					string.Compare(cn, @"ntext", StringComparison.InvariantCultureIgnoreCase) == 0 ||
					//string.Compare(cn, @"varchar", StringComparison.InvariantCultureIgnoreCase) == 0 ||
					string.Compare(cn, @"text", StringComparison.InvariantCultureIgnoreCase) == 0;
			}
			else
			{
				return false;
			}
		}

		private void fillObjectToControls()
		{
			var table = _row.Table;

			foreach (DataColumn column in table.Columns)
			{
				var c = findControlByColumn(column);
				setControlValue(c, _row[column]);
			}
		}

		private void fillControlsToObject()
		{
			var table = _row.Table;

			foreach (DataColumn column in table.Columns)
			{
				var c = findControlByColumn(column);
				_row[column] = checkNull(getControlValue(c));
			}
		}

		private static object checkNull(object o)
		{
			if (o == null)
			{
				return DBNull.Value;
			}
			else
			{
				var s = o as string;
				if (s == null)
				{
					return o;
				}
				else if (string.IsNullOrEmpty(s))
				{
					return DBNull.Value;
				}
				else
				{
					return o;
				}
			}
		}

		private void setControlValue(Control control, object value)
		{
			if (control is CheckBox)
			{
				((CheckBox)control).Checked = (value != null && value != DBNull.Value) && (bool)value;
			}
			else if (control is TextBox)
			{
				control.Text = value == null ? null : value.ToString();
			}
			else
			{
				control.Text = value == null ? null : value.ToString();
			}
		}

		private object getControlValue(Control control)
		{
			if (control is CheckBox)
			{
				return ((CheckBox)control).Checked;
			}
			else if (control is TextBox)
			{
				return control.Text;
			}
			else
			{
				return control.Text;
			}
		}

		private Control findControlByColumn(DataColumn column)
		{
			foreach (Control control in panel1.Controls)
			{
				if (!(control is Label))
				{
					var c = control.Tag as DataColumn;
					if (c != null && c.ColumnName == column.ColumnName)
					{
						return control;
					}
				}
			}

			throw new Exception(string.Format("Control for column '{0}' not found.", column.ColumnName));
		}
	}
}